/* Benchmarking helper routines
 * Ben Lynn
 */

/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/* need these for getrusage version of get_time
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
*/

/* need for clock() version of get_time
#include <time.h>
*/

/* need for times() version of get_time
#include <sys/times.h>
#include <unistd.h>
*/

// for get_time_of_day
#include <sys/time.h>

#include <string.h>
#include "benchmark.h"

static struct benchmark_s bmtable[1000];

static int bmcount = 0;

void bm_put(double t, char *s)
{
    int i;
    for (i=0; i<bmcount; i++) {
	if (!strcmp(bmtable[i].name, s)) break;
    }
    if (i == bmcount) bmcount++;

    strcpy(bmtable[i].name, s);
    bmtable[i].time = t;
}

double bm_get(char *s)
{
    int i;
    for (i=0; i<bmcount; i++) {
	if (!strcmp(bmtable[i].name, s)) return bmtable[i].time;
    }
    return 0.0;
}

double bm_get_time(void)
//code taken from Victor Shoup's NTL library
{
    /*
    //getrusage() version
    struct rusage used;

    getrusage(RUSAGE_SELF, &used);
    return (used.ru_utime.tv_sec + used.ru_stime.tv_sec +
	(used.ru_utime.tv_usec + used.ru_stime.tv_usec) / 1e6);
    */
    /*
   //clock() version
    static clock_t last_clock = 0;
    static double acc = 0;

    clock_t this_clock;
    double delta;

    this_clock = clock();

    if (last_clock == 0) {
	delta = 0;
    } else {
	delta = (this_clock - last_clock)/((double)CLOCKS_PER_SEC);
	if (delta < 0) delta = 0;
    }

    acc += delta;
    last_clock = this_clock;

    return acc;
    */

    /* times() version
    static clock_t last_clock = 0;
    static double acc = 0;
    struct tms used;

    clock_t this_clock;
    double delta;

    times(&used);
    this_clock = used.tms_utime + used.tms_stime;

    if (last_clock == 0) {
	delta = 0;
    } else {
	delta = (this_clock - last_clock)/((double)sysconf(_SC_CLK_TCK));
	if (delta < 0) delta = 0;
    }

    acc += delta;
    last_clock = this_clock;

    return acc;
    */

    static struct timeval last_tv, tv;
    static int first = 1;
    static double res = 0;

    if (first) {
	gettimeofday(&last_tv, NULL);
	first = 0;
	return 0;
    } else {
	gettimeofday(&tv, NULL);
	res += tv.tv_sec - last_tv.tv_sec;
	res += (tv.tv_usec - last_tv.tv_usec) / 1000000.0;
	last_tv = tv;

	return res;
    }
}
