
    #include <stdio.h>
    #include <stdlib.h>
    #include <unistd.h>
    #include <errno.h>
    #include <string.h>
    #include <netdb.h>
    #include <sys/types.h>
    #include <netinet/in.h>
    #include <sys/socket.h>
    #include "ibe.h"
    #include "format.h"
    #include "ibe_progs.h"

    CONF_CTX *cnfctx;

    int portnum;

    #define MAXDATASIZE 1000 // max number of bytes we can get at once 

    int main(int argc, char *argv[])
    {
	char defaultcnffile[] = "client.cnf";
	char tempfile[] = "temp.cli";
	char *cnffile = defaultcnffile;
	char *paramsfile;
	char *receivefile;
	int status;
        int sockfd, numbytes;  
        char buf[MAXDATASIZE];
        struct hostent *he;
        struct sockaddr_in their_addr; // connector's address information 

	byte_string_t M;
	char *id;
	unsigned char *buftemp;
	int pbufsize = 100;
	char *idarray[2];
	unsigned char *ptext;
	int ptextlen;
	char filebuf[MAXDATASIZE];
	int count;
	FILE *fp;

	printf("\nIBE-SSL test client v1.0\nby J. Adam Sowers (jasowers@virginia.edu)\n");
	printf("based on Stanford IBE 0.21\n\n");

        if (argc != 3) // need the server's DNS name and file to encrypt
	{ 
	    fprintf(stderr,"usage: %s hostname filename\n", argv[0]);
	    fprintf(stderr, "The client will automatically encrypt with the hostname provided.\n\n");
            exit(1);
        }

	printf("Loading config file...\n");
        cnfctx = LoadConfig(cnffile);

        if (!cnfctx)
	{
	    fprintf(stderr, "error opening %s\n", cnffile);
	    exit(1);
        }

	portnum = GetIntParam(cnfctx, "port", 0, 31832);
        paramsfile = GetPathParam(cnfctx, "params", 0, "params.txt");

        IBE_init();
        status = FMT_load_params(paramsfile);
        if (status != 1) 
        {
	    fprintf(stderr, "error loading params file %s\n", paramsfile);
	    exit(1);
        }

	// get the host info 
        if ((he=gethostbyname(argv[1])) == NULL)
	{  
            perror("gethostbyname");
            exit(1);
        }

	// set up the socket
        if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("socket");
            exit(1);
        }

	// standard TCP/IP socket parameters
        their_addr.sin_family = AF_INET;    // host byte order 
        their_addr.sin_port = htons(portnum);  // short, network byte order 
        their_addr.sin_addr = *((struct in_addr *)he->h_addr);
        memset(&(their_addr.sin_zero), '\0', 8);  // zero the rest of the struct 

	printf("Connecting to %s...\n", argv[1]);

	// connecto to socket
        if (connect(sockfd, (struct sockaddr *)&their_addr,sizeof(struct sockaddr)) == -1) 
	{
            perror("connect");
            exit(1);
        }

	// receive server header string
        if ((numbytes=recv(sockfd, buf, MAXDATASIZE-1, 0)) == -1) 
	{
       	    perror("recv");
       	    exit(1);
       	}

       	buf[numbytes] = '\0';
       	printf("Received: %s",buf);

	// make IBE id from argument specified
	id = IBE_make_id(argv[1], NULL);

	printf("opening %s...\n",  argv[2]);
	fp = fopen(argv[2], "r");
	if(!fp)
	{
 	    printf("error opening file %s.\n", argv[2]);
	    close(sockfd);
	    return 1;
	}	

	printf("encrypting file with public key %s...\n", argv[1]);
	ptext = (unsigned char *) malloc(pbufsize);
	ptextlen = 0;

	// read in contents of file, put in buffer to encrypt
	for(;;)
	{
	    if (feof(fp)) break;
	    fgets(filebuf, 256, fp);
	    count = 0;
	    while((filebuf[count] != '\0') && !feof(fp))
	    {
		ptext[ptextlen] = filebuf[count];
		count++;
		ptextlen++;
		if (ptextlen >= pbufsize) 
		{
			pbufsize *= 2;
		    buftemp = (unsigned char *) malloc(pbufsize);
		    memcpy(buftemp, ptext, ptextlen);
		    free(ptext);
		    ptext = buftemp;
		}
	    }
	}

	// close original file
	fclose(fp);
	printf("Encryption successful. Sending encrypted message...\n");

	idarray[0] = id;
	idarray[1] = NULL;
	M->data = ptext;
	M->len = ptextlen;

	// open encrypted file for writing
	fp = fopen(tempfile, "w");

	fprintf(fp, "\n-----BEGIN IBE-----\n");
	FMT_encrypt(fp, M, idarray);
	fprintf(fp, "-----END IBE-----\n");

	byte_string_clear(M);

	fflush(fp);
	rewind(fp);
	fclose(fp);

	// reopen the encrypted file to read in and send to server
	fp = fopen(tempfile, "r");
	fgets(filebuf, 256, fp);
	while(!feof(fp))
	{
	    send(sockfd, filebuf, 256, 0);
	    fgets(filebuf, 256, fp);
	} 

	fclose(fp);
	remove(tempfile);

	printf("Done sending message.\nReceving unencrypted message...\n");
	
	numbytes = 1;

	receivefile = argv[2];
	strcat(receivefile, ".out");

	fp = fopen(receivefile, "w");

	// receive server's response (unencrypted message; should be the same as
	//the original file.
	while(numbytes) 
	{
	    if ((numbytes = recv(sockfd, buf, MAXDATASIZE - 1, 0)) == -1) 
	    {
		perror("recv");
       		exit(1);
	    }

 	    buf[numbytes] = '\0';

	    if(strcmp(buf, "!!!disconnect!!!") == 0)
	    {
		numbytes = 0;
		printf("\nDone receving message. Server closing connection...");
	    }
	    else
	       	fprintf(fp, "%s", buf);
	}
	
	fflush(fp);
	fclose(fp);
	printf(" done. \nProgram exiting...\n");

	// all done, close the socket and exit
	close(sockfd);
        return 0;
    } 
