/* for testing; extracts a key share given a server's master share
 * (users should never have access to any master share)
 * also constructs any private key given
 * the share files (which in real life would never be together in one place)
 * Ben Lynn
 */

/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdlib.h>
#include <openssl/evp.h>
#include "format.h"
#include "ibe_progs.h"

int extract_share(int argc, char **argv)
{
    char *id;
    char *sfile;
    byte_string_t mshare, kshare;
    char *name, *header;

    if (argc < 3) {
	fprintf(stderr, "Usage: extract_share ID SHAREFILE\n");
	return 1;
    }

    id = IBE_make_id(argv[1], NULL);

    sfile = argv[2];
    FMT_load_byte_string(sfile, &name, &header, mshare);

    IBE_load_master_share(mshare);
    IBE_extract_share(kshare, id);

    FMT_crypt_save_fp(stdout, kshare, "password");

    return 0;
}

int key_from_master_shares(int argc, char **argv)
{
    char *id;
    int count = IBE_threshold();
    char **sfiles;
    int i;
    char *privkeyfile;
    byte_string_t key;

    if (argc < 3) {
	fprintf(stderr, "Usage: construct_private ID SHAREFILES...\n");
	return 1;
    }

    if (argc - 2 < count) {
	fprintf(stderr, "not enough pieces to make a private key\n");
	return 1;
    }

    id = IBE_make_id(argv[1], NULL);

    sfiles = (char **) malloc(count * sizeof(char*));
    for (i=0; i<count; i++) {
	sfiles[i] = argv[i+2];
    }

    privkeyfile = GetStringParam(cnfctx, "keyfile", 0, "keyfile");

    if (!FMT_construct_master(sfiles, count)) {
	fprintf(stderr, "at least one bad share file!\n");
	return 1;
    }
    IBE_extract(key, id);

    FMT_crypt_save(privkeyfile, key, "password");

    return 0;
}

int fixed_password_decrypt(int argc, char **argv)
{
    char *privkeyfile = GetStringParam(cnfctx, "keyfile", 0, "keyfile");
    byte_string_t key;
    byte_string_t M;
    int status;

    status = FMT_crypt_load(privkeyfile, key, "password");
    if (status != 1) {
	fprintf(stderr, "error loading private key %s\n", privkeyfile);
	return 1;
    }

    status = FMT_decrypt(M, stdin, key);
    if (status != 1) {
	fprintf(stderr, "error in decryption\n");
	return 1;
    }

    byte_string_printf(M, "%c");
    return 0;
}
