#include <string.h>
#include <openssl/pem.h>
#include <openssl/rand.h>
#include "ibe.h"
#include "format.h"

static const char serverstr[] = "Server";

static void mime_encode(byte_string_t R, byte_string_t M)
//R = encoded(M)
{
    int outl, l;
    EVP_ENCODE_CTX ctx;

    byte_string_init(R, 10 * M->len);

    EVP_EncodeInit(&ctx);
    EVP_EncodeUpdate(&ctx,R->data,&outl,M->data,M->len);
    EVP_EncodeFinal(&ctx,&R->data[outl],&l);
    outl+=l;
    byte_string_reinit(R, outl);
}

int FMT_load_byte_string(const char *filename,
    char **name, char **header, byte_string_t bs)
//load a PEM encoded byte-string
{
    unsigned char *data;
    long len;
    FILE *fp;
    int status;

    fp = fopen(filename, "r");
    if (!fp) return 0;

    status = PEM_read(fp, name, header, &data, &len);
    if (status != 1) return status;
    bs->data = data;
    bs->len = len;
    fclose(fp);

    return 1;
}

int FMT_save_byte_string(const char *filename, char *name,
	char *header, byte_string_t bs)
//save a PEM encoded byte-string
{
    int status;
    FILE *fp;
    fp = fopen(filename, "w");
    if (!fp) return 0;

    status = PEM_write(fp, name, header, bs->data, bs->len);
    //if (status != 1) return status; //why isn't this ever 1?
    fclose(fp);
    return 1;
}

int FMT_load_raw_byte_string(byte_string_t bs, char *file)
//load a raw byte string from a file
{
    int max = 1024;
    int len = 0;
    FILE *fp;

    fp = fopen(file, "r");
    if (!fp) {
	return 0;
    }
    byte_string_init(bs, max);

    for(;;) {
	len += fread(&bs->data[len], 1, max - len, fp);
	if (len == max) {
	    max *= 2;
	    byte_string_reinit(bs, max);
	} else {
	    if (!feof(fp)) {
		fclose(fp);
		byte_string_clear(bs);
		return 0;
	    }
	    fclose(fp);
	    break;
	}
    }
    byte_string_reinit(bs, len);
    return 1;
}

int FMT_split_master(char **fname, int t, int n)
{
    int i;
    byte_string_t mshare[n];

    IBE_split_master(mshare, t, n);
    for (i=0; i<n; i++) {
	if (1 != FMT_save_byte_string(fname[i], "SHARE", "", mshare[i])) {
	    fprintf(stderr, "error writing to %s\n", fname[i]);
	    return 0;
	}
    }
    return 1;
}

int FMT_construct_master(char **fname, int t)
{
    int i;
    char *name, *header;
    byte_string_t mshare[t];

    for (i=0; i<t; i++) {
	if (1 != FMT_load_byte_string(fname[i], &name, &header, mshare[i])) {
	    fprintf(stderr, "error loading %s\n", fname[i]);
	    return 0;
	}
	//XXX: how to free name and header?
    }
    IBE_construct_master(mshare, t);
    return 1;
}

int FMT_save_params(char *filename)
{
    int status;
    byte_string_t bs;
    FILE *fp;

    IBE_serialize_params(bs);
    fp = fopen(filename, "w");
    if (!fp) {
	fprintf(stderr, "error opening file for write\n");
	return 0;
    }

    status = PEM_write(fp, "PARAMS", "", bs->data, bs->len);
    //if (status != 1) return status;
    fclose(fp);
    byte_string_clear(bs);
    return 1;
}

int FMT_load_params(char *filename)
{
    char *name, *header;
    unsigned char *data;
    long len;
    FILE *fp;
    int status;
    byte_string_t bs;

    fp = fopen(filename, "r");
    if (!fp) return 0;

    status = PEM_read(fp, &name, &header, &data, &len);
    if (status != 1) return status;
    bs->data = data;
    bs->len = len;
    fclose(fp);

    status = IBE_deserialize_params(bs);
    //XXX: clear buffer?
    return 1;
}

int FMT_output_uvw(FILE *stream, char *ver, byte_string_t U,
	byte_string_t *Varray, byte_string_t W, char **idarray)
{
    int count;
    byte_string_t bstmp;

    FILE *fp = stream;
    if (!fp) {
	fp = stdout;
    }

    fprintf(fp, "Version: %s\r\n", ver);
    mime_encode(bstmp, U);
    fprintf(fp, "U: ");
    byte_string_fprintf(fp, bstmp, "%c");
    byte_string_clear(bstmp);
    fprintf(fp, "\r\n");
    for (count=0; idarray[count]; count++) {
	fprintf(fp, "ID: %s\r\n", idarray[count]);

	fprintf(fp, "V: ");
	mime_encode(bstmp, Varray[count]);
	byte_string_fprintf(fp, bstmp, "%c");
	byte_string_clear(bstmp);
	fprintf(fp,"\r\n");
    }
    fprintf(fp, "W: ");
    mime_encode(bstmp, W);
    byte_string_fprintf(fp, bstmp, "%c");
    byte_string_clear(bstmp);
    fprintf(fp, "\r\n");

    return 1;
}

int FMT_encrypt(FILE *outfp, byte_string_t M, char **idarray)
{
    byte_string_t U;
    byte_string_t *Varray;
    byte_string_t W;
    int i, count;
    int status;

    for (count=0; idarray[count]; count++);
    if (count == 0) return 0;

    Varray = (byte_string_t *) malloc(count * sizeof(byte_string_t));

    IBE_encrypt(U, Varray, W, M, idarray);

    status = FMT_output_uvw(outfp, IBE_version(), U, Varray, W, idarray);

    byte_string_clear(U);
    for (i=0; i<count; i++) {
	byte_string_clear(Varray[i]);
    }
    free(Varray);
    byte_string_clear(W);

    return status;
}

int FMT_decrypt(byte_string_t M, FILE *infp, byte_string_t key)
{
    //XXX:possible buffer overflows
    //needs to be rewritten
    byte_string_t U, V, W;
    char line[100];
    int status;

infp = fopen("temp.svr", "r");

    byte_string_init(U, 1000);
    byte_string_init(V, 100);
    byte_string_init(W, 10000);

    for(;;) {
	fgets(line, 100, infp);
	if (feof(infp)) {
	    return(0);
	}
	if (!strncmp(line, "U: ", 3)) {
	    char *s = &line[3];
	    int l, l2;
	    EVP_ENCODE_CTX ctx;
	    EVP_DecodeInit(&ctx);
	    l = 0;
	    for (;;) {
		EVP_DecodeUpdate(&ctx, &U->data[l], &l2, (unsigned char *) s, strlen(s));
		l += l2;
		fgets(line, 100, infp);
		s = line;
		if (feof(infp) || strlen(s) < 5) break;
	    }
	    EVP_DecodeFinal(&ctx, &U->data[l], &l2);
	    U->len = l + l2;
	} else if (!strncmp(line, "V: ", 3)) {
	    char *s = &line[3];
	    int l, l2;
	    EVP_ENCODE_CTX ctx;
	    EVP_DecodeInit(&ctx);
	    EVP_DecodeUpdate(&ctx, V->data, &l, (unsigned char *) s, strlen(s));
	    EVP_DecodeFinal(&ctx, &V->data[l], &l2);
	    V->len = l + l2;
	    if (V->len!=20) {
		return(0);
	    }
	} else if (!strncmp(line, "W: ", 3)) {
	    char *s = &line[3];
	    int l, l2;
	    EVP_ENCODE_CTX ctx;
	    EVP_DecodeInit(&ctx);
	    l = 0;
	    for (;;) {
		EVP_DecodeUpdate(&ctx, &W->data[l], &l2, (unsigned char *) s, strlen(s));
		l += l2;
		fgets(line, 100, infp);
		s = line;
		if (feof(infp) || strlen(s) < 5)
			break;
	    }
	    EVP_DecodeFinal(&ctx, &W->data[l], &l2);
	    W->len = l + l2;
	    break;
	}
    }

    status = IBE_decrypt(M, U, V, W, key);
    if (status != 1) {
	fprintf(stderr, "error in decryption\n");
    }
    return(status);
}

int FMT_crypt_save_fp(FILE *fp, byte_string_t bs, const char *password)
{
    unsigned char key[EVP_MAX_KEY_LENGTH];
    EVP_CIPHER* sharecipher = EVP_des_ede3_cbc();
    EVP_CIPHER_CTX ctx;
    byte_string_t out;
    int outl;
    int saltlen;
    int i;
    int status;

    EVP_BytesToKey(sharecipher, EVP_md5(), NULL,  (unsigned char *) password,
	    strlen(password), 1, key, NULL);
    
    saltlen = EVP_CIPHER_iv_length(sharecipher);

    byte_string_init(out, bs->len + EVP_CIPHER_block_size(sharecipher) + saltlen);

    status = RAND_bytes(out->data, saltlen);
    if (status != 1) return status;

    EVP_EncryptInit(&ctx, sharecipher, key, out->data);
    outl = saltlen;
    EVP_EncryptUpdate(&ctx, &out->data[outl], &i, bs->data, bs->len);
    outl += i;
    EVP_EncryptFinal(&ctx, &out->data[outl], &i);
    outl += i;

    status = PEM_write(fp, "PRIVATE", "", (unsigned char *) out->data, outl);

    byte_string_clear(out);

    return status;
}

int FMT_crypt_save(const char *fname, byte_string_t bs, const char *password)
{
    FILE *fp;
    int status;

    fp = fopen(fname, "w");
    if (!fp) return 0;
    status = FMT_crypt_save_fp(fp, bs, password);

    fclose(fp);

    return status;
}

int FMT_crypt_load(const char *fname, byte_string_t bs, const char *password)
{
    char *name, *header;
    unsigned char *data;
    long len;
    int leni;
    FILE *fp;
    int status;
    EVP_CIPHER_CTX ctx;
    EVP_CIPHER* sharecipher = EVP_des_ede3_cbc();
    unsigned char key[EVP_MAX_KEY_LENGTH];
    int i, l;

    fp = fopen(fname, "r");
    if (!fp) return 0;
    status = PEM_read(fp, &name, &header, &data, &len);

    EVP_BytesToKey(sharecipher, EVP_md5(), NULL, (unsigned char *) password,
	    strlen(password), 1, key, NULL);

    leni = len;

    l = EVP_CIPHER_iv_length(sharecipher);

    byte_string_init(bs, leni + EVP_CIPHER_block_size(sharecipher) - l);

    EVP_DecryptInit(&ctx, sharecipher, key, data);
    EVP_DecryptUpdate(&ctx, bs->data, &i, &data[l], leni - l);
    EVP_DecryptFinal(&ctx, &bs->data[i], &l);
    i += l;
    fclose(fp);

    byte_string_reinit(bs, i);
    return status;
}
