/* header file for format.cc
 * Ben Lynn
 */
/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef FORMAT_H
#define FORMAT_H

#include <stdio.h>
#include "ibe.h"

int FMT_load_params(char *filename);
//load system parameters

int FMT_save_params(char *filename);
//save system parameters

int FMT_split_master(char **fname, int t, int n);
//split the master secret into n files
//(t are needed to recover it)

int FMT_construct_master(char **fname, int t);
//construct master from secret shares
//(shouldn't be used since shares shouldn't be together)

int FMT_encrypt(FILE *outfp, byte_string_t M, char **idarray);
//encrypts message from a buffer for an array of ID's, writes to a file pointer

int FMT_decrypt(byte_string_t M, FILE *infp, byte_string_t key);

int FMT_crypt_save_fp(FILE *fp, byte_string_t bs, const char *password);
int FMT_crypt_save(const char *fname, byte_string_t bs, const char *password);
//save a byte string to an encrypted file
int FMT_crypt_load(const char *fname, byte_string_t bs, const char *password);
//load a byte string from an encrypted file

int FMT_load_byte_string(const char *filename,
    char **name, char **header, byte_string_t bs);
//load a PEM encoded byte-string

int FMT_load_raw_byte_string(byte_string_t bs, char *file);
//load a raw byte string from a file

#endif //FORMAT_H
