/* Generates system parameters.
 * Reads config file from gen.cnf by default
 * Ben Lynn
 */

/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ibe.h"
#include "format.h"
#include "config.h"
#include <stdlib.h>

int main(int argc, char **argv)
{
	char defaultcnffile[] = "gen.cnf";
	char *cnffile;
	CONF_CTX *cnfctx;
	int t, n;
	int bits, qbits;
	char *systemid;
	char *paramsfile;
	char **sharefile;

	char *dfltlist[3];
	char dl0[] = "share";
	char dl1[] = "share2";

	int i;

	if (argc < 2) {
		cnffile = defaultcnffile;
	} else {
		cnffile = argv[1];
	}

	cnfctx = LoadConfig(cnffile);

	if (!cnfctx) {
		fprintf(stderr, "error opening %s\n", cnffile);
		fprintf(stderr, "using default values\n");
		cnfctx = constructCTX();
		//exit(1);
	}

	t = GetIntParam(cnfctx, "threshold", 0, 2);
	n = GetIntParam(cnfctx, "shares", 0, 2);
	bits = GetIntParam(cnfctx, "pbits", 0, 1024);
	qbits = GetIntParam(cnfctx, "qbits", 0, 160);
	systemid = GetStringParam(cnfctx, "system", 0, "noname");
	paramsfile = GetPathParam(cnfctx, "params", 0, "params.txt");
	dfltlist[0] = dl0;
	dfltlist[1] = dl1;
	sharefile = GetListParam(cnfctx, "sharefiles", 0, dfltlist);

	printf("Generating IBE public parameters and secret...\n");
	printf("Parameters:\n");
	printf("system name: %s\n", systemid);
	printf("params file: %s\n", paramsfile);
	printf("%d-out-of-%d sharing\n", t, n);
	printf("%d-bit prime\n", bits);
	printf("%d-bit subgroup\n", qbits);
	printf("share files:\n");
	for (i=0; i<n; i++) {
		if (sharefile[i] == NULL) {
			fprintf(stderr, "not enough filenames given\n");
			exit(1);
		}
		printf("   %s\n", sharefile[i]);
	}

	IBE_init();
	IBE_setup(bits, qbits, systemid);
	FMT_split_master(sharefile, t, n);

	FMT_save_params(paramsfile);

	// test it works
	if (0) {
		byte_string_t key;
		//FMT_construct_master(key, sharefile, t);
		IBE_extract(key, "blynn@cs.stanford.edu");
		byte_string_clear(key);
	}

	return 0;
}
