/* Command-line frontend to IBE system
 * Patches together combine, decrypt, encrypt and request
 * Ben Lynn
 */

/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdlib.h>
#include <string.h>

#include "ibe.h"
#include "format.h"
#include "ibe_progs.h"

CONF_CTX *cnfctx;

int show_params(int argc, char **argv)
{
    IBE_print_params();
    return 1;
}

int main(int argc, char **argv)
{
    char defaultcnffile[] = "ibe.cnf";
    //char defaultcnffile[100];
    char *cnffile = defaultcnffile;
    char *paramsfile;
    int status;
    char *cmd;

    //strcpy(defaultcnffile, getenv("HOME"));
    //strcat(defaultcnffile, "/.ibe/config");

    //XXX: add option to change config file
    if (argc < 2) {
	fprintf(stderr, "Usage: ibe command\n");
	exit(1);
    }
    cnfctx = LoadConfig(cnffile);

    if (!cnfctx) {
	fprintf(stderr, "error opening %s\n", cnffile);
	exit(1);
    }

    paramsfile = GetPathParam(cnfctx, "params", 0, "params.txt");
    IBE_init();
    status = FMT_load_params(paramsfile);
    if (status != 1) {
	fprintf(stderr, "error loading params file %s\n", paramsfile);
	exit(1);
    }

    cmd = argv[1];
    argc--;
    argv++;

    if (!strcmp(cmd, "request")) {
	request(argc, argv);
    } else if (!strcmp(cmd, "decrypt")) {
	decrypt(argc, argv);
    } else if (!strcmp(cmd, "encrypt")) {
	encrypt(argc, argv);
    } else if (!strcmp(cmd, "combine")) {
	combine(argc, argv);
    } else if (!strcmp(cmd, "imratio")) {
	imratio(argc, argv);
    } else if (!strcmp(cmd, "params")) {
	show_params(argc, argv);
    } else if (!strcmp(cmd, "extract_share")) {
	extract_share(argc, argv);
    } else if (!strcmp(cmd, "key_from_master_shares")) {
	key_from_master_shares(argc, argv);
    } else if (!strcmp(cmd, "fixed_password_decrypt")) {
	fixed_password_decrypt(argc, argv);
    } else {
	fprintf(stderr, "unknown command\n");
	return 1;
    }

    return 0;
}
