/* IBE test program
 * possibly one of the most complex hello world programs around
 * Ben Lynn
 */
/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "ibe.h"

int main(void)
{
    char *id;
    char *idarray[2];

    byte_string_t U;
    byte_string_t Varray[2];
    byte_string_t W;
    byte_string_t M;
    byte_string_t key;

    //initialize IBE library
    IBE_init();

    //generate new system parameters:
    //128-bit prime, 64-bit subgroup size
    //and name it "test"
    IBE_setup(128, 64, "test");

    //make an array of ID's to encrypt for
    //(it's a NULL-terminated array of NULL-terminated strings)
    //we only want one (mine!)
    id = IBE_make_id("blynn@stanford.edu", NULL);
    idarray[0] = id;
    idarray[1] = NULL;

    printf("IBE test program\n");
    printf("Test ID: %s\n", id);

    //encrypt the message
    //using the notation in the paper, the encrypted message has the form (U, V, W)
    //IBE_encrypt is designed to encrypt the same message for several ID's
    //at the same time. Thus it takes an (NULL-terminated) array of ID's.
    //It turns out that the U and W is the same for each ID,
    //and only the V's are distinct

    byte_string_init_set(M, "Hello World!\n");

    printf("Encrypting...\n");
    IBE_encrypt(U, Varray, W, M, idarray);

    byte_string_clear(M);

    //print ciphertext in hexadecimal
    printf("U =");
    byte_string_printf(U, " %02X");
    printf("\n");

    printf("V =");
    byte_string_printf(Varray[0], " %02X");
    printf("\n");

    printf("W =");
    byte_string_printf(W, " %02X");
    printf("\n");
    //Extract the private key corresponding to my ID
    //BTW, This is not the normal way of getting a private key;
    //normally the master secret is unknown on the decryption side
    //and they have obtained their private key by fetching it from the PKG

    IBE_extract(key, id);
    //The key is now stored in keytext and has length keytextlen

    //Decrypt the message with my private key
    printf("Decrypting...\n");
    IBE_decrypt(M, U, Varray[0], W, key);

    //Print it out. Hopefully it'll be "Hello, World!\n"
    byte_string_printf(M, "%c");

    return 0;
}
