/* benchmark inversions and multiplications in F_p^2
 * (combines private key shares into a private key)
 * Ben Lynn
 */

/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "benchmark.h"
#include "curve.h"
#include "format.h"
#include "ibe_progs.h"

enum {
    trials = 5000
};

int imratio(int argc, char **argv)
{
    int i;
    fp2_t a[trials], b[trials], c;
    mpz_t x[trials], y[trials], z;
    double t0, t1;
    double R, M, I, S;

    for (i=0; i<trials; i++) {
	fp2_init(a[i]);
	fp2_init(b[i]);
	mpz_init(x[i]);
	mpz_init(y[i]);
    }
    fp2_init(c);

    mpz_init(z);

    printf("F_p^2 operations\n");
    t0 = bm_get_time();
    for (i=0; i<trials; i++) {
	fp2_random(a[i]);
	fp2_random(b[i]);
    }
    t1 = bm_get_time();
    R = t1 - t0;
    printf("%d 2R = %f\n", trials, R);

    t0 = bm_get_time();
    for (i=0; i<trials; i++) {
	fp2_mul(c, a[i], b[i]);
    }
    t1 = bm_get_time();
    M = t1 - t0;
    printf("%d M = %f\n", trials, M);

    t0 = bm_get_time();
    for (i=0; i<trials; i++) {
	fp2_sqr(c, a[i]);
    }
    t1 = bm_get_time();
    S = t1 - t0;
    printf("%d S = %f\n", trials, S);

    t0 = bm_get_time();
    for (i=0; i<trials; i++) {
	fp2_inv(c, a[i]);
    }
    t1 = bm_get_time();
    I = t1 - t0;
    printf("%d I = %f\n", trials, I);

    printf("I/M = %f\n", I/M);

    printf("F_p operations\n");

    t0 = bm_get_time();
    for (i=0; i<trials; i++) {
	mympz_randomp(x[i]);
	mympz_randomp(y[i]);
    }
    t1 = bm_get_time();
    R = t1 - t0;
    printf("%d 2R = %f\n", trials, R);

    t0 = bm_get_time();
    for (i=0; i<trials; i++) {
	mpz_mul(z, x[i], y[i]);
    }
    t1 = bm_get_time();
    M = t1 - t0;
    printf("%d M = %f\n", trials, M);

    t0 = bm_get_time();
    for (i=0; i<trials; i++) {
	zp_invert(z, x[i]);
    }
    t1 = bm_get_time();
    I = t1 - t0;
    printf("%d I = %f\n", trials, I);

    printf("I/M = %f\n", I/M);

    for (i=0; i<trials; i++) {
	fp2_clear(a[i]);
	fp2_clear(b[i]);
	mpz_clear(x[i]);
	mpz_clear(y[i]);
    }
    fp2_clear(c);

    mpz_clear(z);

    return 0;
}
