/* Hack that makes PKCS12 blobs.
 * (We're waiting for OpenSSL to provide library routines.)
 * Based on code by Dan Boneh
 * Ben Lynn
 */
/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*------------------------------------------------------------------------*/
/*--		Generate an IBE PKCS12 blob				--*/
/*------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/ssl.h>
#include <openssl/crypto.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/pkcs12.h>
#include <openssl/rand.h>
#include <openssl/evp.h>
#include <openssl/asn1_mac.h>
#include <openssl/asn1.h>

#define PKCS12_TXT "PKCS12 DATA"

static BIO *bio_err = NULL;
static char* hackbuf;
static int hackbuflen;
/*------------------------------------------------------------------------*/
/*--	Convert IBE struc from internal format to exportable format	--*/
/*------------------------------------------------------------------------*/

/*
int i2d_hack(char *a, unsigned char **pp)
{
  int size;
  if (pp) {
  	memcpy(*pp, hackbuf, hackbuflen);
  }
  size = hackbuflen;
  printf("returning %d\n", size);
  return size;
}
*/


/*------------------------------------------------------------------------*/
/*--	Function taken from OpenSSL to encrypt PKCS7 structure		--*/
/*------------------------------------------------------------------------*/

PKCS7 *PKCS12_pack_p7encdata_IBE (int pbe_nid, const char *pass, int passlen,
             unsigned char *salt, int saltlen, int iter, char *bag)
{
	ASN1_OCTET_STRING *oct;
  PKCS7 *p7;
  X509_ALGOR *pbe;

  if (!(p7 = PKCS7_new())) {
    PKCS12err(PKCS12_F_PKCS12_PACK_P7ENCDATA, ERR_R_MALLOC_FAILURE);
    return NULL;
  }
  if(!PKCS7_set_type(p7, NID_pkcs7_encrypted)) {
    PKCS12err(PKCS12_F_PKCS12_PACK_P7ENCDATA,
	      PKCS12_R_ERROR_SETTING_ENCRYPTED_DATA_TYPE);
    return NULL;
  }
  if (!(pbe = PKCS5_pbe_set (pbe_nid, iter, salt, saltlen))) {
    PKCS12err(PKCS12_F_PKCS12_PACK_P7ENCDATA, ERR_R_MALLOC_FAILURE);
    return NULL;
  }
  X509_ALGOR_free(p7->d.encrypted->enc_data->algorithm);
  p7->d.encrypted->enc_data->algorithm = pbe;
  M_ASN1_OCTET_STRING_free(p7->d.encrypted->enc_data->enc_data);
/*
  if (!(p7->d.encrypted->enc_data->enc_data =
        PKCS12_i2d_encrypt (pbe, i2d_hack, pass, passlen,
			    (char *)bag, 0))) {
    PKCS12err(PKCS12_F_PKCS12_PACK_P7ENCDATA, PKCS12_R_ENCRYPT_ERROR);
    return NULL;
  }
  */
	if (!(oct = M_ASN1_OCTET_STRING_new ())) {
		PKCS12err(PKCS12_F_PKCS12_I2D_ENCRYPT,ERR_R_MALLOC_FAILURE);
		return NULL;
	}
	if (!PKCS12_pbe_crypt (pbe, pass, passlen, hackbuf, hackbuflen, &oct->data,
				 &oct->length, 1)) {
		PKCS12err(PKCS12_F_PKCS12_I2D_ENCRYPT,PKCS12_R_ENCRYPT_ERROR);
		return NULL;
	}
  if (!(p7->d.encrypted->enc_data->enc_data = oct)) {
    PKCS12err(PKCS12_F_PKCS12_PACK_P7ENCDATA, PKCS12_R_ENCRYPT_ERROR);
    return NULL;
  }
  
  return p7;
}

/*------------------------------------------------------------------------*/
/*--	Generate a new PKCS12 IBE blob					--*/
/*------------------------------------------------------------------------*/

int pkcs12_write(unsigned char **outbuf, int *outbuflen,
		unsigned char *inbuf, int inbuflen, char *pass) {
  STACK *safes = NULL;
  PKCS7 *authsafe = NULL;
  PKCS12 *p12 = NULL;
  int cert_pbe = NID_pbe_WithSHA1And3_Key_TripleDES_CBC;
  int iter = PKCS12_DEFAULT_ITER;
  int maciter = PKCS12_DEFAULT_ITER;
  int err = -1;
  unsigned char salt[4];
  BIO *membio;
  BUF_MEM *bm;
  int j;
  BIO *out;
  //temp fix for no /dev/urandom
  //char asdf[2000];
  //RAND_seed(asdf, 2000);
  
  hackbuf = inbuf;
  hackbuflen = inbuflen;

  if (bio_err == NULL ) 
    bio_err = BIO_new_fp (stderr, BIO_NOCLOSE);

  out = BIO_new_fp (stdout, BIO_NOCLOSE);

  /*-- Read password from console by which PKCS12 will be encrypted  --*/

  /*
  if (EVP_read_pw_string(pass, 50, "Enter Export Password: ", 1)) {
    BIO_printf (bio_err, "Can't read Password\n");
    goto end;
  }
  BIO_printf(bio_err, "\n");
  */

  /*-- Convert key structure into encrypted authsafe --*/
  if ((RAND_bytes(salt, sizeof(salt)) < 1)) {
    fprintf(stderr, "Error generating random salt.\n");
    goto end;
  }

  authsafe = PKCS12_pack_p7encdata_IBE(cert_pbe, pass, -1, 
				       salt, sizeof(salt),
				       iter, NULL);
  if (!authsafe) {
    ERR_print_errors (bio_err);
    goto end;
  }

  /*-- Create PKCS12 structure --*/

  safes = sk_new (NULL);
  sk_push (safes, (char *)authsafe);

  p12 = PKCS12_init (NID_pkcs7_data);
  
  M_PKCS12_pack_authsafes (p12, safes);
  
  PKCS12_set_mac (p12, pass, -1, NULL, 0, maciter, NULL);
  

  /*-- Write PKCS12 structure --*/

membio = BIO_new(BIO_s_mem());
if (!membio) {
  BIO_printf(bio_err, "BIO_new: error\n");
  goto end;
}

  i2d_PKCS12_bio (membio, p12);
  
    bm = (BUF_MEM *)membio->ptr;
	*outbuf = (unsigned char *) malloc(bm->length * sizeof(unsigned char));
	*outbuflen = bm->length;
	for (j=0; j<*outbuflen; j++) {
		(*outbuf)[j] = (bm->data)[j];
	}

  /*-- Output PEM formatted blob --*/
/*
    bm = (BUF_MEM *)membio->ptr;
    EVP_EncodeInit(&ectx);
    EVP_EncodeUpdate(&ectx, buf, &j, (unsigned char *) bm->data, bm->length);
    len = j;
    EVP_EncodeFinal(&ectx, buf+len, &j);
    len += j;
    
    BIO_printf(out, "-----BEGIN %s-----\n", PKCS12_TXT);
    BIO_write(out, buf, len);
    BIO_printf(out, "-----END %s-----\n", PKCS12_TXT);
	*/

  err = 0;

end:
  if (safes) sk_pop_free(safes, PKCS7_free);
  if (membio) BIO_free(membio);
  if (p12) PKCS12_free(p12);

  //memset(pass, 0, sizeof(pass));

  if (err)
    ERR_print_errors (bio_err);

  return err;
}


/*------------------------------------------------------------------------*/
/*--	Parse the key blob						--*/
/*------------------------------------------------------------------------*/

int pkcs12_read(unsigned char** out, int *outl,
		const unsigned char* in, const int inl, const char *pass)
{
  PKCS12 *p12 = NULL;
  STACK *asafes = NULL;
  PKCS7 *p7 = NULL;
  int bagnid;
	ASN1_OCTET_STRING *oct;
	int err = 0;
	unsigned char *buf;
	int buflen;

  /*-- Unpack PKCS12 structure --*/

  if (!(p12 = d2i_PKCS12(NULL, (unsigned char **) &in, inl))) {
    ERR_print_errors(bio_err);
    goto end;
  }

  /*-- Verify MAC --*/

  if (!PKCS12_verify_mac(p12, pass, -1)) {
    BIO_printf (bio_err, "Mac verify error: invalid password?\n");
    goto end;
  } 

  /*-- Get all safes from bag.       --*/
  /*-- There should be only one safe --*/

  if (!(asafes = M_PKCS12_unpack_authsafes(p12))) {
    BIO_printf (bio_err, "Improper PKCS12 bag.\n");
    goto end;
  }

  if (sk_num(asafes) != 1) {
    BIO_printf(bio_err, "Too many safes in structure.\n");
    goto end;
  } 
 
  p7 = (PKCS7 *) sk_value (asafes, 0);	/* Parse the one and only safe */
  bagnid = OBJ_obj2nid (p7->type);

  if (bagnid != NID_pkcs7_encrypted) {
    BIO_printf(bio_err, "Invalid PKCS7 bag type.\n");
    goto end;
  }

  /*
  key = (IBE_KEY_STRUC *)PKCS12_decrypt_d2i(
		     (p7)->d.encrypted->enc_data->algorithm,
		     (char *(*)())d2i_PKCS12_IBE, NULL,
		     pass, -1, 
		     (p7)->d.encrypted->enc_data->enc_data, 2);
			 */

	oct = (p7)->d.encrypted->enc_data->enc_data;

	if (!PKCS12_pbe_crypt ((p7)->d.encrypted->enc_data->algorithm,
			 pass, -1, oct->data, oct->length,
			&buf, &buflen, 0)) {
		PKCS12err(PKCS12_F_PKCS12_DECRYPT_D2I,PKCS12_R_PKCS12_PBE_CRYPT_ERROR);
		return 0;
	}
	*out = buf;
	*outl = buflen;

end:
  if (asafes) sk_pop_free (asafes, PKCS7_free); 
  if (p12) PKCS12_free(p12);  

  return err;
}
