/* Tests Weil pairing code
 * (check by eye that the output is indeed bilinear)
 * Ben Lynn
 */
/*
Copyright (C) 2001 Benjamin Lynn (blynn@cs.stanford.edu)

This file is part of the Stanford IBE system.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "curve.h"

static mpz_t p, q;
static mpz_t p1onq;
static mpz_t p1;
static point_t P;
static point_t P1, P2, Psum;
static point_t Q;
static fp2_t r, r1, r2, r3, r4;

void test(void);

int main(int argc, char **argv)
{
    int i, prime;
    if (argc > 1) {
	sscanf(argv[1], "%d", &prime);
    } else prime = 59;

    mpz_init(p);

    mpz_set_ui(p, prime);
    /*
    if (prime % 3 != 2 || !mpz_probab_prime_p(p, 40) || prime == 2) {
	fprintf(stderr, "only works with odd prime p = 2 mod 3\n");
	exit(1);
    }
    */
    if (prime % 12 != 11 || !mpz_probab_prime_p(p, 40) || prime == 2) {
	fprintf(stderr, "only works with odd prime p = 11 mod 12\n");
	exit(1);
    }

    mpz_init(p1);
    mpz_init(p1onq);
    mpz_init(q);
    mpz_add_ui(p1, p, 1);
    //mpz_set_ui(p1onq, 6);
    mpz_set_ui(p1onq, 12);
    mpz_divexact(q, p1, p1onq);

    curve_init_prime(p, q);

    point_init(P);
    point_init(P1);
    point_init(P2);
    point_init(Psum);
    point_init(Q);
    fp2_init(r);
    fp2_init(r1);
    fp2_init(r2);

    for (i=0; i<10; i++) test();
    return 0;
}

void test(void)
{
    //simple test
    general_point_random(P);
    general_point_mul(P1, p1onq, P);

    general_point_random(P);
    general_point_mul(P2, p1onq, P);

    general_point_random(P);
    general_point_mul(Q, p1onq, P);

    point_add(Psum, P1, P2);

    printf("random P1: ");
    fp2_out_str(stdout, 0, P1->x);
    printf(" ");
    fp2_out_str(stdout, 0, P1->y);
    printf("\n");

    printf("random P2: ");
    fp2_out_str(stdout, 0, P2->x);
    printf(" ");
    fp2_out_str(stdout, 0, P2->y);
    printf("\n");

    printf("random Q: ");
    fp2_out_str(stdout, 0, Q->x);
    printf(" ");
    fp2_out_str(stdout, 0, Q->y);
    printf("\n");

    weil_pairing(r, P1, P1);
    printf("e(P1, P1) = ");
    fp2_out_str(stdout, 0, r);
    printf("\n");

    weil_pairing(r1, P1, Q);
    printf("e(P1, Q) = ");
    fp2_out_str(stdout, 0, r1);
    printf("\n");

    weil_pairing(r2, P2, Q);
    printf("e(P2, Q) = ");
    fp2_out_str(stdout, 0, r2);
    printf("\n");

    fp2_mul(r, r1, r2);
    printf("e(P1, Q)e(P2, Q) = ");
    fp2_out_str(stdout, 0, r);
    printf("\n");

    weil_pairing(r, Psum, Q);
    printf("e(P1 + P2, Q) = ");
    fp2_out_str(stdout, 0, r);
    printf("\n");

    weil_pairing(r3, Q, P1);
    printf("e(Q, P1) = ");
    fp2_out_str(stdout, 0, r3);
    printf("\n");

    weil_pairing(r4, Q, P2);
    printf("e(Q, P2) = ");
    fp2_out_str(stdout, 0, r4);
    printf("\n");

    fp2_mul(r, r3, r4);
    printf("e(Q, P1)e(Q, P2) = ");
    fp2_out_str(stdout, 0, r);
    printf("\n");

    weil_pairing(r, Q, Psum);
    printf("e(Q, P1 + P2) = ");
    fp2_out_str(stdout, 0, r);
    printf("\n");

    fp2_mul(r, r1, r3);
    printf("e(P1, Q)e(Q, P1) = ");
    fp2_out_str(stdout, 0, r);
    printf("\n");

    fp2_mul(r, r2, r4);
    printf("e(P2, Q)e(Q, P2) = ");
    fp2_out_str(stdout, 0, r);
    printf("\n");
    mpz_out_str(stdout, 0, p1);
    printf("th powers should be 1\n");

    fp2_pow(r, r1, p1);
    fp2_out_str(stdout, 0, r);
    printf("\n");
    fp2_pow(r, r2, p1);
    fp2_out_str(stdout, 0, r);
    printf("\n");
    fp2_pow(r, r3, p1);
    fp2_out_str(stdout, 0, r);
    printf("\n");
    fp2_pow(r, r4, p1);
    fp2_out_str(stdout, 0, r);
    printf("\n");
}
